/* ***** BEGIN LICENSE BLOCK *****
 * Licensed under Version: MPL 1.1/GPL 2.0/LGPL 2.1
 * Full Terms at http://mozile.mozdev.org/license.html
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Playsophy code (www.playsophy.com).
 *
 * The Initial Developer of the Original Code is Playsophy
 * Portions created by the Initial Developer are Copyright (C) 2002-2003
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * ***** END LICENSE BLOCK ***** */

/**
@fileoverview
 * domlevel3.js
 *
 * Early DOM Level 3 implementation - part of: 
 * - the Validation module: http://www.w3.org/TR/2003/WD-DOM-Level-3-Val-20030205/
 * - the load and save module: http://www.w3.org/TR/2003/WD-DOM-Level-3-LS-20030226/
 * Both modules are designed for building editors.
 *
 * Note: this JS will go away once these modules are properly supported within standard
 * browsers.
 *
 * POST05:
 * - properly pretty print XML in save methods
 * - implement documentURI and other level 3 core methods
*/

/************************* Core *****************************************/	

/************************* Load and Save ********************************/

/**
 * saveXML method from DocumentLS
 * see: http://www.w3.org/TR/2003/WD-DOM-Level-3-LS-20030226/load-save.html#LS-DocumentLS
 */
/* Temporarily removed : not used at the moment.
Document.prototype.saveXML = documentSaveXML;
function documentSaveXML(snode)
{
	if(!snode)
		snode = document; // null means document as a whole

	//create a new XMLSerializer
	var objXMLSerializer = new XMLSerializer;
    
	//get the XML string
	var strXML = objXMLSerializer.serializeToString(snode);
    
	//return the XML string
	return strXML;
}*/

/* Temporarily removed : not used at the moment.
Node.prototype.saveXML = function()
{
	//create a new XMLSerializer
	var objXMLSerializer = new XMLSerializer;
    
	//get the XML string
	var strXML = objXMLSerializer.serializeToString(this);
    
	//return the XML string
	return strXML;
}*/

/**
 * markupContent attribute from ElementLS
 *
 * see: http://www.w3.org/TR/2003/WD-DOM-Level-3-LS-20030226/load-save.html#LS-ElementLS
 */
/* Temporarily removed : not used at the moment.
Element.prototype.__defineGetter__(
	"markupContent",
	function()
	{
		var objXMLSerializer = new XMLSerializer;
		var strXML = objXMLSerializer.serializeToString(this);
		return strXML;
	}
);*/

/************************* Validation Module ****************************/

/**
allowedChildren attribute from ElementEditVAL.

<p>note: implementation only works for XHTML for now  

<p>Listing X/HTML elements - of course an XML may have others but
we can't know that without reading the DTD/Schema.
 
<p>see: http://www.w3.org/TR/2003/WD-DOM-Level-3-Val-20030205/validation.html#VAL-Interfaces-ElementEditVAL

<p>POST04: change to return list of allowed children ie/ return empty list of string names

@return true if allowed children.
@type boolean
*/
Element.prototype.__defineGetter__(
	"allowedChildren",
	function()
	{
		return !this.isXHTMLEmptyContentType();
	}
);

/*
Document Object Model (DOM) Level 3 Validation Specification
Version 1.0
W3C Recommendation 27 January 2004
http://www.w3.org/TR/DOM-Level-3-Val/Overview.html
http://www.w3.org/TR/DOM-Level-3-Val/validation.html
VAL_ANY_CONTENTTYPE
    The content model contains unordered child information item(s),
		i.e., element, processing instruction, unexpanded entity reference, character,
		and comment information items as defined in the XML Information Set.
		If the schema is a DTD, this corresponds to the ANY content model. 
VAL_ELEMENTS_CONTENTTYPE
    The content model contains a sequence of element information items optionally
		separated by whitespace. If the schema is a DTD, this is the element content
		content model; and if the schema is a W3C XML schema, this is the element-only content type. 
VAL_EMPTY_CONTENTTYPE
    The content model does not allow any content. If the schema is a W3C XML schema,
		this corresponds to the empty content type; and if the schema is a DTD,
		this corresponds to the EMPTY content model. 
VAL_MIXED_CONTENTTYPE
    The content model contains a sequence of ordered element information items
		optionally interspersed with character data. If the schema is a W3C XML schema,
		this corresponds to the mixed content type. 
VAL_SIMPLE_CONTENTTYPE
    The content model contains character information items.
		If the schema is a W3C XML schema, then the element has a content type
		of VAL_SIMPLE_CONTENTTYPE if the type of the element is a simple type definition,
		or the type of the element is a complexType whose {content type} is a simple type definition.
*/

/**
0.
@final
*/
Element.EMPTY_CONTENTTYPE = 0;
/**
1.
@final
*/
Element.SIMPLE_CONTENTTYPE = 1;
/**
2.
@final
*/
Element.ANY_CONTENTTYPE = 2;
/**
3.
@final
*/
Element.MIXED_CONTENTTYPE = 3;
/**
4.
@final
*/
Element.ELEMENTS_CONTENTTYPE = 4;
/**
contentType from ElementEditVAL.
 
<p>Can be used for walking editable text. 
 
<p>see: http://www.w3.org/TR/2003/WD-DOM-Level-3-Val-20030205/validation.html#VAL-Interfaces-ElementEditVAL-ElementEditVAL-contentType

<p>Note: specification doesn't seem to cover its return values properly.

@return contentType (0-4)
@type number
 */
Element.prototype.__defineGetter__(
	"contentType",
	function()
	{
		// non childbearing or just has no children
		if(this.childNodes.length == 0)	
		{
			if(this.isXHTMLEmptyContentType())
				return Element.EMPTY_CONTENTTYPE;
			else
				return Element.ANY_CONTENTTYPE;
		}

		// only a text node as a child
		if((this.childNodes.length == 1) && (this.childNodes[0].nodeType == Node.TEXT_NODE))
			return Element.SIMPLE_CONTENTTYPE;

		for(var i=0; i<this.childNodes.length; i++) {
			if(this.childNodes[i].nodeType == Node.TEXT_NODE)
				return Element.MIXED_CONTENTTYPE;
		}

		return Element.ELEMENTS_CONTENTTYPE;
	}
);

/**
isWhitespaceOnly attribute from CharacterDataEditVAL.
 
<p>see: http://www.w3.org/TR/2003/WD-DOM-Level-3-Val-20030205/validation.html#VAL-Interfaces-CharacterDataEditVAL-isWhitespaceOnly

<p>Reg expression \S is equivalent to : [^ \f\n\r\t\u00A0\u2028\u2029] (See Javascript Manual)

@return true if empty node or whitespace.
@type boolean
*/
Text.prototype.__defineGetter__(
	"isWhitespaceOnly",
	function()
	{
		if(this.nodeValue.length == 0) // empty node: interpreting as whitespace?
			return true;
		if(/\S+/.test(this.nodeValue)) // any non white space visible characters
			return false;
		return true;
	}
);